# Generated by Django 2.2.17 on 2021-05-27 10:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_webstories', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='externalstory',
            name='url_hash',
            field=models.CharField(db_index=True, editable=False, max_length=40, unique=True),
        ),
    ]
